import 'package:flutter/material.dart';
import 'package:vgkprogaming/theme/vgk_shared_theme.dart';
import 'package:vgkprogaming/widget/vgk_alert_widget/vgk_alert_style.dart';
import 'package:vgkprogaming/widget/vgk_alert_widget/vgk_animation_transition.dart';
import 'package:vgkprogaming/widget/vgk_alert_widget/vgk_constants.dart';
import 'package:vgkprogaming/widget/vgk_alert_widget/vgk_dialog_button.dart';

class VgkAlertWidget {
  final BuildContext context;
  final AlertType? type;
  final VgkAlertStyle style;
  final Image? image;
  final String title;
  final String? desc;
  final Widget? content;
  final List<VgkDialogButton>? buttons;
  final Function? closeFunction;

  VgkAlertWidget({
    required this.context,
    this.type,
    this.style = const VgkAlertStyle(),
    this.image,
    required this.title,
    this.desc,
    this.content,
    this.buttons,
    this.closeFunction,
  });

  Future<Object?> show(primaryColor) async {
    return await showGeneralDialog(
      context: context,
      pageBuilder: (BuildContext buildContext, Animation<double> animation,
          Animation<double> secondaryAnimation) {
        return _buildDialog(primaryColor);
      },
      barrierDismissible: style.isOverlayTapDismiss,
      barrierLabel: MaterialLocalizations.of(context).modalBarrierDismissLabel,
      barrierColor: style.overlayColor,
      transitionDuration: style.animationDuration,
      transitionBuilder: (
        BuildContext context,
        Animation<double> animation,
        Animation<double> secondaryAnimation,
        Widget child,
      ) =>
          _showAnimation(animation, secondaryAnimation, child),
    );
  }

  Widget _buildDialog(primaryColor) {
    return Center(
      child: ConstrainedBox(
        constraints: style.constraints!,
        child: Center(
          child: SingleChildScrollView(
            child: AlertDialog(
              backgroundColor: style.backgroundColor,
              shape: style.alertBorder,
              titlePadding: const EdgeInsets.all(0.0),
              title: Center(
                child: Column(
                  mainAxisAlignment: MainAxisAlignment.center,
                  children: <Widget>[
                    _getCloseButton(),
                    Padding(
                      padding: EdgeInsets.fromLTRB(
                          20, (style.isCloseButton ? 0 : 20), 20, 0),
                      child: Column(
                        children: <Widget>[
                          _getImage(),
                          const SizedBox(
                            height: 15,
                          ),
                          Text(
                            title,
                            style: style.titleStyle,
                            textAlign: TextAlign.center,
                          ),
                          SizedBox(
                            height: desc == null ? 5 : 10,
                          ),
                          if (desc == null)
                            Container()
                          else
                            Text(
                              desc!,
                              style: style.descStyle,
                              textAlign: TextAlign.center,
                            ),
                          content!,
                        ],
                      ),
                    )
                  ],
                ),
              ),
              contentPadding: style.buttonAreaPadding,
              content: Row(
                mainAxisAlignment: MainAxisAlignment.center,
                children: _getButtons(primaryColor),
              ),
            ),
          ),
        ),
      ),
    );
  }

  Widget _getCloseButton() {
    return style.isCloseButton
        ? GestureDetector(
            onTap: () {
              Navigator.pop(context);
              closeFunction!();
            },
            child: Padding(
              padding: const EdgeInsets.fromLTRB(0, 10, 10, 0),
              child: Container(
                alignment: FractionalOffset.topRight,
                child: const SizedBox(
                  width: 20,
                  height: 20,
                  child: Icon(
                    Icons.close,
                    size: 24,
                  ),
                ),
              ),
            ),
          )
        : Container();
  }

  List<Widget> _getButtons(primaryColor) {
    List<Widget> expandedButtons = [];
    if (buttons != null) {
      for (var button in buttons!) {
        var buttonWidget = Padding(
          padding: const EdgeInsets.only(left: 2, right: 2),
          child: button,
        );
        if (buttons?.length == 1) {
          expandedButtons.add(buttonWidget);
        } else {
          expandedButtons.add(Expanded(
            child: buttonWidget,
          ));
        }
      }
    } else {
      expandedButtons.add(
        Expanded(
          child: VgkDialogButton(
            child: Text(
              "CANCEL",
              style: VgkProGamingTheme.h1(primaryColor),
            ),
            onPressed: () => Navigator.pop(context),
          ),
        ),
      );
    }

    return expandedButtons;
  }

  Widget _getImage() {
    Widget response = image ?? Container();
    switch (type) {
      case AlertType.success:
        response = Image.asset(
          '$kImagePath/icon_success.png',
          package: 'rflutter_alert',
        );
        break;
      case null:
        response = Image.asset(
          '$kImagePath/icon_success.png',
          package: 'rflutter_alert',
        );
        break;
      case AlertType.error:
        response = Image.asset(
          '$kImagePath/icon_error.png',
          package: 'rflutter_alert',
        );
        break;
      case AlertType.info:
        response = Image.asset(
          '$kImagePath/icon_info.png',
          package: 'rflutter_alert',
        );
        break;
      case AlertType.warning:
        response = Image.asset(
          '$kImagePath/icon_warning.png',
          package: 'rflutter_alert',
        );
        break;
      case AlertType.none:
        response = Container();
        break;
    }
    return response;
  }

  _showAnimation(animation, secondaryAnimation, child) {
    if (style.animationType == AnimationType.fromRight) {
      return VgkAnimationTransition.fromRight(
          animation, secondaryAnimation, child);
    } else if (style.animationType == AnimationType.fromLeft) {
      return VgkAnimationTransition.fromLeft(
          animation, secondaryAnimation, child);
    } else if (style.animationType == AnimationType.fromBottom) {
      return VgkAnimationTransition.fromBottom(
          animation, secondaryAnimation, child);
    } else if (style.animationType == AnimationType.grow) {
      return VgkAnimationTransition.grow(animation, secondaryAnimation, child);
    } else if (style.animationType == AnimationType.shrink) {
      return VgkAnimationTransition.shrink(
          animation, secondaryAnimation, child);
    } else {
      return VgkAnimationTransition.fromTop(
          animation, secondaryAnimation, child);
    }
  }
}
